# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/jobs.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10proto/jobs.proto\x12\x04jobs\"%\n\x04Lock\x12\x0c\n\x04peer\x18\x01 \x01(\t\x12\x0f\n\x07\x63reated\x18\x02 \x01(\x04\"Z\n\x03Job\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05owner\x18\x02 \x01(\t\x12\x18\n\x04lock\x18\x03 \x01(\x0b\x32\n.jobs.Lock\x12\x10\n\x08protocol\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\"\x10\n\x0eGetJobsRequest\"\'\n\rSetJobRequest\x12\x16\n\x03job\x18\x01 \x01(\x0b\x32\t.jobs.Job\"\x1e\n\x10\x44\x65leteJobRequest\x12\n\n\x02id\x18\x01 \x01(\t\"\x1f\n\x11\x41\x63quireJobRequest\x12\n\n\x02id\x18\x01 \x01(\t\"\x1f\n\x11ReleaseJobRequest\x12\n\n\x02id\x18\x01 \x01(\t\".\n\x13JobMutationResponse\x12\x17\n\x04jobs\x18\x01 \x03(\x0b\x32\t.jobs.JobB.Z,github.com/smartin015/peerprint/pubsub/protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.jobs_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/smartin015/peerprint/pubsub/proto'
  _LOCK._serialized_start=26
  _LOCK._serialized_end=63
  _JOB._serialized_start=65
  _JOB._serialized_end=155
  _GETJOBSREQUEST._serialized_start=157
  _GETJOBSREQUEST._serialized_end=173
  _SETJOBREQUEST._serialized_start=175
  _SETJOBREQUEST._serialized_end=214
  _DELETEJOBREQUEST._serialized_start=216
  _DELETEJOBREQUEST._serialized_end=246
  _ACQUIREJOBREQUEST._serialized_start=248
  _ACQUIREJOBREQUEST._serialized_end=279
  _RELEASEJOBREQUEST._serialized_start=281
  _RELEASEJOBREQUEST._serialized_end=312
  _JOBMUTATIONRESPONSE._serialized_start=314
  _JOBMUTATIONRESPONSE._serialized_end=360
# @@protoc_insertion_point(module_scope)
